# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['InstanceConfigurationArgs', 'InstanceConfiguration']

@pulumi.input_type
class InstanceConfigurationArgs:
    def __init__(__self__, *,
                 compartment_id: pulumi.Input[str],
                 defined_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 freeform_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 instance_details: Optional[pulumi.Input['InstanceConfigurationInstanceDetailsArgs']] = None,
                 instance_id: Optional[pulumi.Input[str]] = None,
                 source: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a InstanceConfiguration resource.
        :param pulumi.Input[str] compartment_id: The OCID of the compartment containing the instance. Instances created from instance configurations are placed in the same compartment as the instance that was used to create the instance configuration.
        :param pulumi.Input[Mapping[str, Any]] defined_tags: Defined tags for this resource. Each key is predefined and scoped to a namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Operations.CostCenter": "42"}`
        :param pulumi.Input[str] display_name: A user-friendly name. Does not have to be unique, and it's changeable. Avoid entering confidential information.
        :param pulumi.Input[Mapping[str, Any]] freeform_tags: Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Department": "Finance"}`
        :param pulumi.Input[str] instance_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the instance to use to create the instance configuration.
        :param pulumi.Input[str] source: The source of the instance configuration. An instance configuration defines the settings to use when creating Compute instances, including details such as the base image, shape, and metadata. You can also specify the associated resources for the instance, such as block volume attachments and network configuration.
               
               When you create an instance configuration using an existing instance as a template, the instance configuration does not include any information from the source instance's boot volume, such as installed applications, binaries, and files on the instance. It also does not include the contents of any block volumes that are attached to the instance.
               
               To create an instance configuration that includes the custom setup from an instance's boot volume, you must first create a custom image from the instance (see [CreateImage](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Image/CreateImage)). Then, use the custom image to launch a new instance (see [LaunchInstance](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Instance/LaunchInstance)). Finally, create the instance configuration based on the instance that you created from the custom image.
               
               To include block volume contents with an instance configuration, first create a backup of the attached block volumes (see [CreateVolumeBackup](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/VolumeBackup/CreateVolumeBackup)). Then, create the instance configuration by specifying the list of settings, using [InstanceConfigurationVolumeSourceFromVolumeBackupDetails](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/datatypes/InstanceConfigurationVolumeSourceFromVolumeBackupDetails) to include the block volume backups in the list of settings.
               
               The following values are supported:
        """
        pulumi.set(__self__, "compartment_id", compartment_id)
        if defined_tags is not None:
            pulumi.set(__self__, "defined_tags", defined_tags)
        if display_name is not None:
            pulumi.set(__self__, "display_name", display_name)
        if freeform_tags is not None:
            pulumi.set(__self__, "freeform_tags", freeform_tags)
        if instance_details is not None:
            pulumi.set(__self__, "instance_details", instance_details)
        if instance_id is not None:
            pulumi.set(__self__, "instance_id", instance_id)
        if source is not None:
            pulumi.set(__self__, "source", source)

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> pulumi.Input[str]:
        """
        The OCID of the compartment containing the instance. Instances created from instance configurations are placed in the same compartment as the instance that was used to create the instance configuration.
        """
        return pulumi.get(self, "compartment_id")

    @compartment_id.setter
    def compartment_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "compartment_id", value)

    @property
    @pulumi.getter(name="definedTags")
    def defined_tags(self) -> Optional[pulumi.Input[Mapping[str, Any]]]:
        """
        Defined tags for this resource. Each key is predefined and scoped to a namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Operations.CostCenter": "42"}`
        """
        return pulumi.get(self, "defined_tags")

    @defined_tags.setter
    def defined_tags(self, value: Optional[pulumi.Input[Mapping[str, Any]]]):
        pulumi.set(self, "defined_tags", value)

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> Optional[pulumi.Input[str]]:
        """
        A user-friendly name. Does not have to be unique, and it's changeable. Avoid entering confidential information.
        """
        return pulumi.get(self, "display_name")

    @display_name.setter
    def display_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "display_name", value)

    @property
    @pulumi.getter(name="freeformTags")
    def freeform_tags(self) -> Optional[pulumi.Input[Mapping[str, Any]]]:
        """
        Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Department": "Finance"}`
        """
        return pulumi.get(self, "freeform_tags")

    @freeform_tags.setter
    def freeform_tags(self, value: Optional[pulumi.Input[Mapping[str, Any]]]):
        pulumi.set(self, "freeform_tags", value)

    @property
    @pulumi.getter(name="instanceDetails")
    def instance_details(self) -> Optional[pulumi.Input['InstanceConfigurationInstanceDetailsArgs']]:
        return pulumi.get(self, "instance_details")

    @instance_details.setter
    def instance_details(self, value: Optional[pulumi.Input['InstanceConfigurationInstanceDetailsArgs']]):
        pulumi.set(self, "instance_details", value)

    @property
    @pulumi.getter(name="instanceId")
    def instance_id(self) -> Optional[pulumi.Input[str]]:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the instance to use to create the instance configuration.
        """
        return pulumi.get(self, "instance_id")

    @instance_id.setter
    def instance_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "instance_id", value)

    @property
    @pulumi.getter
    def source(self) -> Optional[pulumi.Input[str]]:
        """
        The source of the instance configuration. An instance configuration defines the settings to use when creating Compute instances, including details such as the base image, shape, and metadata. You can also specify the associated resources for the instance, such as block volume attachments and network configuration.

        When you create an instance configuration using an existing instance as a template, the instance configuration does not include any information from the source instance's boot volume, such as installed applications, binaries, and files on the instance. It also does not include the contents of any block volumes that are attached to the instance.

        To create an instance configuration that includes the custom setup from an instance's boot volume, you must first create a custom image from the instance (see [CreateImage](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Image/CreateImage)). Then, use the custom image to launch a new instance (see [LaunchInstance](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Instance/LaunchInstance)). Finally, create the instance configuration based on the instance that you created from the custom image.

        To include block volume contents with an instance configuration, first create a backup of the attached block volumes (see [CreateVolumeBackup](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/VolumeBackup/CreateVolumeBackup)). Then, create the instance configuration by specifying the list of settings, using [InstanceConfigurationVolumeSourceFromVolumeBackupDetails](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/datatypes/InstanceConfigurationVolumeSourceFromVolumeBackupDetails) to include the block volume backups in the list of settings.

        The following values are supported:
        """
        return pulumi.get(self, "source")

    @source.setter
    def source(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "source", value)


@pulumi.input_type
class _InstanceConfigurationState:
    def __init__(__self__, *,
                 compartment_id: Optional[pulumi.Input[str]] = None,
                 deferred_fields: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 defined_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 freeform_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 instance_details: Optional[pulumi.Input['InstanceConfigurationInstanceDetailsArgs']] = None,
                 instance_id: Optional[pulumi.Input[str]] = None,
                 source: Optional[pulumi.Input[str]] = None,
                 time_created: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering InstanceConfiguration resources.
        :param pulumi.Input[str] compartment_id: The OCID of the compartment containing the instance. Instances created from instance configurations are placed in the same compartment as the instance that was used to create the instance configuration.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] deferred_fields: Parameters that were not specified when the instance configuration was created, but that are required to launch an instance from the instance configuration. See the [LaunchInstanceConfiguration](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Instance/LaunchInstanceConfiguration) operation.
        :param pulumi.Input[Mapping[str, Any]] defined_tags: Defined tags for this resource. Each key is predefined and scoped to a namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Operations.CostCenter": "42"}`
        :param pulumi.Input[str] display_name: A user-friendly name. Does not have to be unique, and it's changeable. Avoid entering confidential information.
        :param pulumi.Input[Mapping[str, Any]] freeform_tags: Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Department": "Finance"}`
        :param pulumi.Input[str] instance_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the instance to use to create the instance configuration.
        :param pulumi.Input[str] source: The source of the instance configuration. An instance configuration defines the settings to use when creating Compute instances, including details such as the base image, shape, and metadata. You can also specify the associated resources for the instance, such as block volume attachments and network configuration.
               
               When you create an instance configuration using an existing instance as a template, the instance configuration does not include any information from the source instance's boot volume, such as installed applications, binaries, and files on the instance. It also does not include the contents of any block volumes that are attached to the instance.
               
               To create an instance configuration that includes the custom setup from an instance's boot volume, you must first create a custom image from the instance (see [CreateImage](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Image/CreateImage)). Then, use the custom image to launch a new instance (see [LaunchInstance](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Instance/LaunchInstance)). Finally, create the instance configuration based on the instance that you created from the custom image.
               
               To include block volume contents with an instance configuration, first create a backup of the attached block volumes (see [CreateVolumeBackup](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/VolumeBackup/CreateVolumeBackup)). Then, create the instance configuration by specifying the list of settings, using [InstanceConfigurationVolumeSourceFromVolumeBackupDetails](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/datatypes/InstanceConfigurationVolumeSourceFromVolumeBackupDetails) to include the block volume backups in the list of settings.
               
               The following values are supported:
        :param pulumi.Input[str] time_created: The date and time the instance configuration was created, in the format defined by [RFC3339](https://tools.ietf.org/html/rfc3339).  Example: `2016-08-25T21:10:29.600Z`
        """
        if compartment_id is not None:
            pulumi.set(__self__, "compartment_id", compartment_id)
        if deferred_fields is not None:
            pulumi.set(__self__, "deferred_fields", deferred_fields)
        if defined_tags is not None:
            pulumi.set(__self__, "defined_tags", defined_tags)
        if display_name is not None:
            pulumi.set(__self__, "display_name", display_name)
        if freeform_tags is not None:
            pulumi.set(__self__, "freeform_tags", freeform_tags)
        if instance_details is not None:
            pulumi.set(__self__, "instance_details", instance_details)
        if instance_id is not None:
            pulumi.set(__self__, "instance_id", instance_id)
        if source is not None:
            pulumi.set(__self__, "source", source)
        if time_created is not None:
            pulumi.set(__self__, "time_created", time_created)

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> Optional[pulumi.Input[str]]:
        """
        The OCID of the compartment containing the instance. Instances created from instance configurations are placed in the same compartment as the instance that was used to create the instance configuration.
        """
        return pulumi.get(self, "compartment_id")

    @compartment_id.setter
    def compartment_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "compartment_id", value)

    @property
    @pulumi.getter(name="deferredFields")
    def deferred_fields(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Parameters that were not specified when the instance configuration was created, but that are required to launch an instance from the instance configuration. See the [LaunchInstanceConfiguration](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Instance/LaunchInstanceConfiguration) operation.
        """
        return pulumi.get(self, "deferred_fields")

    @deferred_fields.setter
    def deferred_fields(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "deferred_fields", value)

    @property
    @pulumi.getter(name="definedTags")
    def defined_tags(self) -> Optional[pulumi.Input[Mapping[str, Any]]]:
        """
        Defined tags for this resource. Each key is predefined and scoped to a namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Operations.CostCenter": "42"}`
        """
        return pulumi.get(self, "defined_tags")

    @defined_tags.setter
    def defined_tags(self, value: Optional[pulumi.Input[Mapping[str, Any]]]):
        pulumi.set(self, "defined_tags", value)

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> Optional[pulumi.Input[str]]:
        """
        A user-friendly name. Does not have to be unique, and it's changeable. Avoid entering confidential information.
        """
        return pulumi.get(self, "display_name")

    @display_name.setter
    def display_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "display_name", value)

    @property
    @pulumi.getter(name="freeformTags")
    def freeform_tags(self) -> Optional[pulumi.Input[Mapping[str, Any]]]:
        """
        Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Department": "Finance"}`
        """
        return pulumi.get(self, "freeform_tags")

    @freeform_tags.setter
    def freeform_tags(self, value: Optional[pulumi.Input[Mapping[str, Any]]]):
        pulumi.set(self, "freeform_tags", value)

    @property
    @pulumi.getter(name="instanceDetails")
    def instance_details(self) -> Optional[pulumi.Input['InstanceConfigurationInstanceDetailsArgs']]:
        return pulumi.get(self, "instance_details")

    @instance_details.setter
    def instance_details(self, value: Optional[pulumi.Input['InstanceConfigurationInstanceDetailsArgs']]):
        pulumi.set(self, "instance_details", value)

    @property
    @pulumi.getter(name="instanceId")
    def instance_id(self) -> Optional[pulumi.Input[str]]:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the instance to use to create the instance configuration.
        """
        return pulumi.get(self, "instance_id")

    @instance_id.setter
    def instance_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "instance_id", value)

    @property
    @pulumi.getter
    def source(self) -> Optional[pulumi.Input[str]]:
        """
        The source of the instance configuration. An instance configuration defines the settings to use when creating Compute instances, including details such as the base image, shape, and metadata. You can also specify the associated resources for the instance, such as block volume attachments and network configuration.

        When you create an instance configuration using an existing instance as a template, the instance configuration does not include any information from the source instance's boot volume, such as installed applications, binaries, and files on the instance. It also does not include the contents of any block volumes that are attached to the instance.

        To create an instance configuration that includes the custom setup from an instance's boot volume, you must first create a custom image from the instance (see [CreateImage](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Image/CreateImage)). Then, use the custom image to launch a new instance (see [LaunchInstance](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Instance/LaunchInstance)). Finally, create the instance configuration based on the instance that you created from the custom image.

        To include block volume contents with an instance configuration, first create a backup of the attached block volumes (see [CreateVolumeBackup](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/VolumeBackup/CreateVolumeBackup)). Then, create the instance configuration by specifying the list of settings, using [InstanceConfigurationVolumeSourceFromVolumeBackupDetails](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/datatypes/InstanceConfigurationVolumeSourceFromVolumeBackupDetails) to include the block volume backups in the list of settings.

        The following values are supported:
        """
        return pulumi.get(self, "source")

    @source.setter
    def source(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "source", value)

    @property
    @pulumi.getter(name="timeCreated")
    def time_created(self) -> Optional[pulumi.Input[str]]:
        """
        The date and time the instance configuration was created, in the format defined by [RFC3339](https://tools.ietf.org/html/rfc3339).  Example: `2016-08-25T21:10:29.600Z`
        """
        return pulumi.get(self, "time_created")

    @time_created.setter
    def time_created(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "time_created", value)


class InstanceConfiguration(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 compartment_id: Optional[pulumi.Input[str]] = None,
                 defined_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 freeform_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 instance_details: Optional[pulumi.Input[pulumi.InputType['InstanceConfigurationInstanceDetailsArgs']]] = None,
                 instance_id: Optional[pulumi.Input[str]] = None,
                 source: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        This resource provides the Instance Configuration resource in Oracle Cloud Infrastructure Core service.

        Creates an instance configuration. An instance configuration is a template that defines the
        settings to use when creating Compute instances.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_oci as oci

        test_instance_configuration = oci.core.InstanceConfiguration("testInstanceConfiguration",
            compartment_id=var["compartment_id"],
            defined_tags={
                "Operations.CostCenter": "42",
            },
            display_name=var["instance_configuration_display_name"],
            freeform_tags={
                "Department": "Finance",
            },
            instance_details=oci.core.InstanceConfigurationInstanceDetailsArgs(
                instance_type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                block_volumes=[oci.core.InstanceConfigurationInstanceDetailsBlockVolumeArgs(
                    attach_details=oci.core.InstanceConfigurationInstanceDetailsBlockVolumeAttachDetailsArgs(
                        type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        device=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_pv_encryption_in_transit_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_read_only=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_shareable=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        use_chap=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    create_details=oci.core.InstanceConfigurationInstanceDetailsBlockVolumeCreateDetailsArgs(
                        autotune_policies=[oci.core.InstanceConfigurationInstanceDetailsBlockVolumeCreateDetailsAutotunePolicyArgs(
                            autotune_type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                            max_vpus_per_gb=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        )],
                        availability_domain=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        backup_policy_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        compartment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        defined_tags={
                            "Operations.CostCenter": "42",
                        },
                        display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        freeform_tags={
                            "Department": "Finance",
                        },
                        kms_key_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        size_in_gbs=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        source_details=oci.core.InstanceConfigurationInstanceDetailsBlockVolumeCreateDetailsSourceDetailsArgs(
                            type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                            id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        ),
                        vpus_per_gb=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    volume_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                )],
                launch_details=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsArgs(
                    agent_config=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsAgentConfigArgs(
                        are_all_plugins_disabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_management_disabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_monitoring_disabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        plugins_configs=[oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsAgentConfigPluginsConfigArgs(
                            desired_state=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                            name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        )],
                    ),
                    availability_config=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsAvailabilityConfigArgs(
                        recovery_action=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    availability_domain=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    capacity_reservation_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    compartment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    create_vnic_details=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsCreateVnicDetailsArgs(
                        assign_private_dns_record=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        assign_public_ip=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        defined_tags={
                            "Operations.CostCenter": "42",
                        },
                        display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        freeform_tags={
                            "Department": "Finance",
                        },
                        hostname_label=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        nsg_ids=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        private_ip=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        skip_source_dest_check=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        subnet_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    dedicated_vm_host_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    defined_tags={
                        "Operations.CostCenter": "42",
                    },
                    display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    extended_metadata=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    fault_domain=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    freeform_tags={
                        "Department": "Finance",
                    },
                    instance_options=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsInstanceOptionsArgs(
                        are_legacy_imds_endpoints_disabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    ipxe_script=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    is_pv_encryption_in_transit_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    launch_mode=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    launch_options=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsLaunchOptionsArgs(
                        boot_volume_type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        firmware=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_consistent_volume_naming_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_pv_encryption_in_transit_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        network_type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        remote_data_volume_type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    metadata=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    platform_config=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsPlatformConfigArgs(
                        type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        are_virtual_instructions_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_access_control_service_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_input_output_memory_management_unit_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_measured_boot_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_memory_encryption_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_secure_boot_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_symmetric_multi_threading_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_trusted_platform_module_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        numa_nodes_per_socket=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        percentage_of_cores_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    preemptible_instance_config=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsPreemptibleInstanceConfigArgs(
                        preemption_action=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsPreemptibleInstanceConfigPreemptionActionArgs(
                            type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                            preserve_boot_volume=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        ),
                    ),
                    preferred_maintenance_action=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    shape=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    shape_config=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsShapeConfigArgs(
                        baseline_ocpu_utilization=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        memory_in_gbs=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        nvmes=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        ocpus=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    source_details=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsSourceDetailsArgs(
                        source_type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        boot_volume_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        boot_volume_size_in_gbs=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        boot_volume_vpus_per_gb=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        image_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                ),
                secondary_vnics=[oci.core.InstanceConfigurationInstanceDetailsSecondaryVnicArgs(
                    create_vnic_details=oci.core.InstanceConfigurationInstanceDetailsSecondaryVnicCreateVnicDetailsArgs(
                        assign_private_dns_record=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        assign_public_ip=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        defined_tags={
                            "Operations.CostCenter": "42",
                        },
                        display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        freeform_tags={
                            "Department": "Finance",
                        },
                        hostname_label=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        nsg_ids=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        private_ip=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        skip_source_dest_check=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        subnet_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    nic_index=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                )],
            ),
            instance_id=oci_core_instance["test_instance"]["id"],
            source=var["instance_configuration_source"])
        ```

        ## Import

        InstanceConfigurations can be imported using the `id`, e.g.

        ```sh
         $ pulumi import oci:Core/instanceConfiguration:InstanceConfiguration test_instance_configuration "id"
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] compartment_id: The OCID of the compartment containing the instance. Instances created from instance configurations are placed in the same compartment as the instance that was used to create the instance configuration.
        :param pulumi.Input[Mapping[str, Any]] defined_tags: Defined tags for this resource. Each key is predefined and scoped to a namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Operations.CostCenter": "42"}`
        :param pulumi.Input[str] display_name: A user-friendly name. Does not have to be unique, and it's changeable. Avoid entering confidential information.
        :param pulumi.Input[Mapping[str, Any]] freeform_tags: Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Department": "Finance"}`
        :param pulumi.Input[str] instance_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the instance to use to create the instance configuration.
        :param pulumi.Input[str] source: The source of the instance configuration. An instance configuration defines the settings to use when creating Compute instances, including details such as the base image, shape, and metadata. You can also specify the associated resources for the instance, such as block volume attachments and network configuration.
               
               When you create an instance configuration using an existing instance as a template, the instance configuration does not include any information from the source instance's boot volume, such as installed applications, binaries, and files on the instance. It also does not include the contents of any block volumes that are attached to the instance.
               
               To create an instance configuration that includes the custom setup from an instance's boot volume, you must first create a custom image from the instance (see [CreateImage](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Image/CreateImage)). Then, use the custom image to launch a new instance (see [LaunchInstance](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Instance/LaunchInstance)). Finally, create the instance configuration based on the instance that you created from the custom image.
               
               To include block volume contents with an instance configuration, first create a backup of the attached block volumes (see [CreateVolumeBackup](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/VolumeBackup/CreateVolumeBackup)). Then, create the instance configuration by specifying the list of settings, using [InstanceConfigurationVolumeSourceFromVolumeBackupDetails](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/datatypes/InstanceConfigurationVolumeSourceFromVolumeBackupDetails) to include the block volume backups in the list of settings.
               
               The following values are supported:
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: InstanceConfigurationArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        This resource provides the Instance Configuration resource in Oracle Cloud Infrastructure Core service.

        Creates an instance configuration. An instance configuration is a template that defines the
        settings to use when creating Compute instances.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_oci as oci

        test_instance_configuration = oci.core.InstanceConfiguration("testInstanceConfiguration",
            compartment_id=var["compartment_id"],
            defined_tags={
                "Operations.CostCenter": "42",
            },
            display_name=var["instance_configuration_display_name"],
            freeform_tags={
                "Department": "Finance",
            },
            instance_details=oci.core.InstanceConfigurationInstanceDetailsArgs(
                instance_type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                block_volumes=[oci.core.InstanceConfigurationInstanceDetailsBlockVolumeArgs(
                    attach_details=oci.core.InstanceConfigurationInstanceDetailsBlockVolumeAttachDetailsArgs(
                        type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        device=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_pv_encryption_in_transit_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_read_only=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_shareable=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        use_chap=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    create_details=oci.core.InstanceConfigurationInstanceDetailsBlockVolumeCreateDetailsArgs(
                        autotune_policies=[oci.core.InstanceConfigurationInstanceDetailsBlockVolumeCreateDetailsAutotunePolicyArgs(
                            autotune_type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                            max_vpus_per_gb=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        )],
                        availability_domain=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        backup_policy_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        compartment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        defined_tags={
                            "Operations.CostCenter": "42",
                        },
                        display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        freeform_tags={
                            "Department": "Finance",
                        },
                        kms_key_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        size_in_gbs=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        source_details=oci.core.InstanceConfigurationInstanceDetailsBlockVolumeCreateDetailsSourceDetailsArgs(
                            type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                            id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        ),
                        vpus_per_gb=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    volume_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                )],
                launch_details=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsArgs(
                    agent_config=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsAgentConfigArgs(
                        are_all_plugins_disabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_management_disabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_monitoring_disabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        plugins_configs=[oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsAgentConfigPluginsConfigArgs(
                            desired_state=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                            name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        )],
                    ),
                    availability_config=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsAvailabilityConfigArgs(
                        recovery_action=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    availability_domain=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    capacity_reservation_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    compartment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    create_vnic_details=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsCreateVnicDetailsArgs(
                        assign_private_dns_record=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        assign_public_ip=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        defined_tags={
                            "Operations.CostCenter": "42",
                        },
                        display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        freeform_tags={
                            "Department": "Finance",
                        },
                        hostname_label=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        nsg_ids=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        private_ip=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        skip_source_dest_check=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        subnet_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    dedicated_vm_host_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    defined_tags={
                        "Operations.CostCenter": "42",
                    },
                    display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    extended_metadata=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    fault_domain=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    freeform_tags={
                        "Department": "Finance",
                    },
                    instance_options=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsInstanceOptionsArgs(
                        are_legacy_imds_endpoints_disabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    ipxe_script=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    is_pv_encryption_in_transit_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    launch_mode=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    launch_options=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsLaunchOptionsArgs(
                        boot_volume_type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        firmware=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_consistent_volume_naming_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_pv_encryption_in_transit_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        network_type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        remote_data_volume_type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    metadata=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    platform_config=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsPlatformConfigArgs(
                        type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        are_virtual_instructions_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_access_control_service_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_input_output_memory_management_unit_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_measured_boot_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_memory_encryption_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_secure_boot_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_symmetric_multi_threading_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        is_trusted_platform_module_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        numa_nodes_per_socket=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        percentage_of_cores_enabled=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    preemptible_instance_config=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsPreemptibleInstanceConfigArgs(
                        preemption_action=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsPreemptibleInstanceConfigPreemptionActionArgs(
                            type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                            preserve_boot_volume=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        ),
                    ),
                    preferred_maintenance_action=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    shape=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    shape_config=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsShapeConfigArgs(
                        baseline_ocpu_utilization=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        memory_in_gbs=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        nvmes=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        ocpus=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    source_details=oci.core.InstanceConfigurationInstanceDetailsLaunchDetailsSourceDetailsArgs(
                        source_type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        boot_volume_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        boot_volume_size_in_gbs=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        boot_volume_vpus_per_gb=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        image_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                ),
                secondary_vnics=[oci.core.InstanceConfigurationInstanceDetailsSecondaryVnicArgs(
                    create_vnic_details=oci.core.InstanceConfigurationInstanceDetailsSecondaryVnicCreateVnicDetailsArgs(
                        assign_private_dns_record=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        assign_public_ip=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        defined_tags={
                            "Operations.CostCenter": "42",
                        },
                        display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        freeform_tags={
                            "Department": "Finance",
                        },
                        hostname_label=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        nsg_ids=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        private_ip=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        skip_source_dest_check=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                        subnet_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    ),
                    display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    nic_index=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                )],
            ),
            instance_id=oci_core_instance["test_instance"]["id"],
            source=var["instance_configuration_source"])
        ```

        ## Import

        InstanceConfigurations can be imported using the `id`, e.g.

        ```sh
         $ pulumi import oci:Core/instanceConfiguration:InstanceConfiguration test_instance_configuration "id"
        ```

        :param str resource_name: The name of the resource.
        :param InstanceConfigurationArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(InstanceConfigurationArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 compartment_id: Optional[pulumi.Input[str]] = None,
                 defined_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 freeform_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 instance_details: Optional[pulumi.Input[pulumi.InputType['InstanceConfigurationInstanceDetailsArgs']]] = None,
                 instance_id: Optional[pulumi.Input[str]] = None,
                 source: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = InstanceConfigurationArgs.__new__(InstanceConfigurationArgs)

            if compartment_id is None and not opts.urn:
                raise TypeError("Missing required property 'compartment_id'")
            __props__.__dict__["compartment_id"] = compartment_id
            __props__.__dict__["defined_tags"] = defined_tags
            __props__.__dict__["display_name"] = display_name
            __props__.__dict__["freeform_tags"] = freeform_tags
            __props__.__dict__["instance_details"] = instance_details
            __props__.__dict__["instance_id"] = instance_id
            __props__.__dict__["source"] = source
            __props__.__dict__["deferred_fields"] = None
            __props__.__dict__["time_created"] = None
        super(InstanceConfiguration, __self__).__init__(
            'oci:Core/instanceConfiguration:InstanceConfiguration',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            compartment_id: Optional[pulumi.Input[str]] = None,
            deferred_fields: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            defined_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
            display_name: Optional[pulumi.Input[str]] = None,
            freeform_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
            instance_details: Optional[pulumi.Input[pulumi.InputType['InstanceConfigurationInstanceDetailsArgs']]] = None,
            instance_id: Optional[pulumi.Input[str]] = None,
            source: Optional[pulumi.Input[str]] = None,
            time_created: Optional[pulumi.Input[str]] = None) -> 'InstanceConfiguration':
        """
        Get an existing InstanceConfiguration resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] compartment_id: The OCID of the compartment containing the instance. Instances created from instance configurations are placed in the same compartment as the instance that was used to create the instance configuration.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] deferred_fields: Parameters that were not specified when the instance configuration was created, but that are required to launch an instance from the instance configuration. See the [LaunchInstanceConfiguration](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Instance/LaunchInstanceConfiguration) operation.
        :param pulumi.Input[Mapping[str, Any]] defined_tags: Defined tags for this resource. Each key is predefined and scoped to a namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Operations.CostCenter": "42"}`
        :param pulumi.Input[str] display_name: A user-friendly name. Does not have to be unique, and it's changeable. Avoid entering confidential information.
        :param pulumi.Input[Mapping[str, Any]] freeform_tags: Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Department": "Finance"}`
        :param pulumi.Input[str] instance_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the instance to use to create the instance configuration.
        :param pulumi.Input[str] source: The source of the instance configuration. An instance configuration defines the settings to use when creating Compute instances, including details such as the base image, shape, and metadata. You can also specify the associated resources for the instance, such as block volume attachments and network configuration.
               
               When you create an instance configuration using an existing instance as a template, the instance configuration does not include any information from the source instance's boot volume, such as installed applications, binaries, and files on the instance. It also does not include the contents of any block volumes that are attached to the instance.
               
               To create an instance configuration that includes the custom setup from an instance's boot volume, you must first create a custom image from the instance (see [CreateImage](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Image/CreateImage)). Then, use the custom image to launch a new instance (see [LaunchInstance](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Instance/LaunchInstance)). Finally, create the instance configuration based on the instance that you created from the custom image.
               
               To include block volume contents with an instance configuration, first create a backup of the attached block volumes (see [CreateVolumeBackup](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/VolumeBackup/CreateVolumeBackup)). Then, create the instance configuration by specifying the list of settings, using [InstanceConfigurationVolumeSourceFromVolumeBackupDetails](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/datatypes/InstanceConfigurationVolumeSourceFromVolumeBackupDetails) to include the block volume backups in the list of settings.
               
               The following values are supported:
        :param pulumi.Input[str] time_created: The date and time the instance configuration was created, in the format defined by [RFC3339](https://tools.ietf.org/html/rfc3339).  Example: `2016-08-25T21:10:29.600Z`
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _InstanceConfigurationState.__new__(_InstanceConfigurationState)

        __props__.__dict__["compartment_id"] = compartment_id
        __props__.__dict__["deferred_fields"] = deferred_fields
        __props__.__dict__["defined_tags"] = defined_tags
        __props__.__dict__["display_name"] = display_name
        __props__.__dict__["freeform_tags"] = freeform_tags
        __props__.__dict__["instance_details"] = instance_details
        __props__.__dict__["instance_id"] = instance_id
        __props__.__dict__["source"] = source
        __props__.__dict__["time_created"] = time_created
        return InstanceConfiguration(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> pulumi.Output[str]:
        """
        The OCID of the compartment containing the instance. Instances created from instance configurations are placed in the same compartment as the instance that was used to create the instance configuration.
        """
        return pulumi.get(self, "compartment_id")

    @property
    @pulumi.getter(name="deferredFields")
    def deferred_fields(self) -> pulumi.Output[Sequence[str]]:
        """
        Parameters that were not specified when the instance configuration was created, but that are required to launch an instance from the instance configuration. See the [LaunchInstanceConfiguration](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Instance/LaunchInstanceConfiguration) operation.
        """
        return pulumi.get(self, "deferred_fields")

    @property
    @pulumi.getter(name="definedTags")
    def defined_tags(self) -> pulumi.Output[Mapping[str, Any]]:
        """
        Defined tags for this resource. Each key is predefined and scoped to a namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Operations.CostCenter": "42"}`
        """
        return pulumi.get(self, "defined_tags")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> pulumi.Output[str]:
        """
        A user-friendly name. Does not have to be unique, and it's changeable. Avoid entering confidential information.
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter(name="freeformTags")
    def freeform_tags(self) -> pulumi.Output[Mapping[str, Any]]:
        """
        Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Department": "Finance"}`
        """
        return pulumi.get(self, "freeform_tags")

    @property
    @pulumi.getter(name="instanceDetails")
    def instance_details(self) -> pulumi.Output['outputs.InstanceConfigurationInstanceDetails']:
        return pulumi.get(self, "instance_details")

    @property
    @pulumi.getter(name="instanceId")
    def instance_id(self) -> pulumi.Output[str]:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the instance to use to create the instance configuration.
        """
        return pulumi.get(self, "instance_id")

    @property
    @pulumi.getter
    def source(self) -> pulumi.Output[str]:
        """
        The source of the instance configuration. An instance configuration defines the settings to use when creating Compute instances, including details such as the base image, shape, and metadata. You can also specify the associated resources for the instance, such as block volume attachments and network configuration.

        When you create an instance configuration using an existing instance as a template, the instance configuration does not include any information from the source instance's boot volume, such as installed applications, binaries, and files on the instance. It also does not include the contents of any block volumes that are attached to the instance.

        To create an instance configuration that includes the custom setup from an instance's boot volume, you must first create a custom image from the instance (see [CreateImage](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Image/CreateImage)). Then, use the custom image to launch a new instance (see [LaunchInstance](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/Instance/LaunchInstance)). Finally, create the instance configuration based on the instance that you created from the custom image.

        To include block volume contents with an instance configuration, first create a backup of the attached block volumes (see [CreateVolumeBackup](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/VolumeBackup/CreateVolumeBackup)). Then, create the instance configuration by specifying the list of settings, using [InstanceConfigurationVolumeSourceFromVolumeBackupDetails](https://docs.cloud.oracle.com/iaas/api/#/en/iaas/latest/datatypes/InstanceConfigurationVolumeSourceFromVolumeBackupDetails) to include the block volume backups in the list of settings.

        The following values are supported:
        """
        return pulumi.get(self, "source")

    @property
    @pulumi.getter(name="timeCreated")
    def time_created(self) -> pulumi.Output[str]:
        """
        The date and time the instance configuration was created, in the format defined by [RFC3339](https://tools.ietf.org/html/rfc3339).  Example: `2016-08-25T21:10:29.600Z`
        """
        return pulumi.get(self, "time_created")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs

__all__ = [
    'GetNodePoolResult',
    'AwaitableGetNodePoolResult',
    'get_node_pool',
    'get_node_pool_output',
]

@pulumi.output_type
class GetNodePoolResult:
    """
    A collection of values returned by getNodePool.
    """
    def __init__(__self__, cluster_id=None, compartment_id=None, defined_tags=None, freeform_tags=None, id=None, initial_node_labels=None, kubernetes_version=None, lifecycle_details=None, name=None, node_config_details=None, node_eviction_node_pool_settings=None, node_image_id=None, node_image_name=None, node_metadata=None, node_pool_cycling_details=None, node_pool_id=None, node_shape=None, node_shape_configs=None, node_source_details=None, node_sources=None, nodes=None, quantity_per_subnet=None, ssh_public_key=None, state=None, subnet_ids=None):
        if cluster_id and not isinstance(cluster_id, str):
            raise TypeError("Expected argument 'cluster_id' to be a str")
        pulumi.set(__self__, "cluster_id", cluster_id)
        if compartment_id and not isinstance(compartment_id, str):
            raise TypeError("Expected argument 'compartment_id' to be a str")
        pulumi.set(__self__, "compartment_id", compartment_id)
        if defined_tags and not isinstance(defined_tags, dict):
            raise TypeError("Expected argument 'defined_tags' to be a dict")
        pulumi.set(__self__, "defined_tags", defined_tags)
        if freeform_tags and not isinstance(freeform_tags, dict):
            raise TypeError("Expected argument 'freeform_tags' to be a dict")
        pulumi.set(__self__, "freeform_tags", freeform_tags)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if initial_node_labels and not isinstance(initial_node_labels, list):
            raise TypeError("Expected argument 'initial_node_labels' to be a list")
        pulumi.set(__self__, "initial_node_labels", initial_node_labels)
        if kubernetes_version and not isinstance(kubernetes_version, str):
            raise TypeError("Expected argument 'kubernetes_version' to be a str")
        pulumi.set(__self__, "kubernetes_version", kubernetes_version)
        if lifecycle_details and not isinstance(lifecycle_details, str):
            raise TypeError("Expected argument 'lifecycle_details' to be a str")
        pulumi.set(__self__, "lifecycle_details", lifecycle_details)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if node_config_details and not isinstance(node_config_details, list):
            raise TypeError("Expected argument 'node_config_details' to be a list")
        pulumi.set(__self__, "node_config_details", node_config_details)
        if node_eviction_node_pool_settings and not isinstance(node_eviction_node_pool_settings, list):
            raise TypeError("Expected argument 'node_eviction_node_pool_settings' to be a list")
        pulumi.set(__self__, "node_eviction_node_pool_settings", node_eviction_node_pool_settings)
        if node_image_id and not isinstance(node_image_id, str):
            raise TypeError("Expected argument 'node_image_id' to be a str")
        if node_image_id is not None:
            warnings.warn("""The 'node_image_id' field has been deprecated. Please use 'node_source_details' instead. If both fields are specified, then 'node_source_details' will be used.""", DeprecationWarning)
            pulumi.log.warn("""node_image_id is deprecated: The 'node_image_id' field has been deprecated. Please use 'node_source_details' instead. If both fields are specified, then 'node_source_details' will be used.""")

        pulumi.set(__self__, "node_image_id", node_image_id)
        if node_image_name and not isinstance(node_image_name, str):
            raise TypeError("Expected argument 'node_image_name' to be a str")
        if node_image_name is not None:
            warnings.warn("""The 'node_image_name' field has been deprecated. Please use 'node_source_details' instead. If both fields are specified, then 'node_source_details' will be used.""", DeprecationWarning)
            pulumi.log.warn("""node_image_name is deprecated: The 'node_image_name' field has been deprecated. Please use 'node_source_details' instead. If both fields are specified, then 'node_source_details' will be used.""")

        pulumi.set(__self__, "node_image_name", node_image_name)
        if node_metadata and not isinstance(node_metadata, dict):
            raise TypeError("Expected argument 'node_metadata' to be a dict")
        pulumi.set(__self__, "node_metadata", node_metadata)
        if node_pool_cycling_details and not isinstance(node_pool_cycling_details, list):
            raise TypeError("Expected argument 'node_pool_cycling_details' to be a list")
        pulumi.set(__self__, "node_pool_cycling_details", node_pool_cycling_details)
        if node_pool_id and not isinstance(node_pool_id, str):
            raise TypeError("Expected argument 'node_pool_id' to be a str")
        pulumi.set(__self__, "node_pool_id", node_pool_id)
        if node_shape and not isinstance(node_shape, str):
            raise TypeError("Expected argument 'node_shape' to be a str")
        pulumi.set(__self__, "node_shape", node_shape)
        if node_shape_configs and not isinstance(node_shape_configs, list):
            raise TypeError("Expected argument 'node_shape_configs' to be a list")
        pulumi.set(__self__, "node_shape_configs", node_shape_configs)
        if node_source_details and not isinstance(node_source_details, list):
            raise TypeError("Expected argument 'node_source_details' to be a list")
        pulumi.set(__self__, "node_source_details", node_source_details)
        if node_sources and not isinstance(node_sources, list):
            raise TypeError("Expected argument 'node_sources' to be a list")
        pulumi.set(__self__, "node_sources", node_sources)
        if nodes and not isinstance(nodes, list):
            raise TypeError("Expected argument 'nodes' to be a list")
        pulumi.set(__self__, "nodes", nodes)
        if quantity_per_subnet and not isinstance(quantity_per_subnet, int):
            raise TypeError("Expected argument 'quantity_per_subnet' to be a int")
        pulumi.set(__self__, "quantity_per_subnet", quantity_per_subnet)
        if ssh_public_key and not isinstance(ssh_public_key, str):
            raise TypeError("Expected argument 'ssh_public_key' to be a str")
        pulumi.set(__self__, "ssh_public_key", ssh_public_key)
        if state and not isinstance(state, str):
            raise TypeError("Expected argument 'state' to be a str")
        pulumi.set(__self__, "state", state)
        if subnet_ids and not isinstance(subnet_ids, list):
            raise TypeError("Expected argument 'subnet_ids' to be a list")
        pulumi.set(__self__, "subnet_ids", subnet_ids)

    @property
    @pulumi.getter(name="clusterId")
    def cluster_id(self) -> str:
        """
        The OCID of the cluster to which this node pool is attached.
        """
        return pulumi.get(self, "cluster_id")

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> str:
        """
        The OCID of the compartment in which the node pool exists.
        """
        return pulumi.get(self, "compartment_id")

    @property
    @pulumi.getter(name="definedTags")
    def defined_tags(self) -> Mapping[str, Any]:
        """
        Defined tags for this resource. Each key is predefined and scoped to a namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm). Example: `{"Operations.CostCenter": "42"}`
        """
        return pulumi.get(self, "defined_tags")

    @property
    @pulumi.getter(name="freeformTags")
    def freeform_tags(self) -> Mapping[str, Any]:
        """
        Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm). Example: `{"Department": "Finance"}`
        """
        return pulumi.get(self, "freeform_tags")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The OCID of the compute instance backing this node.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="initialNodeLabels")
    def initial_node_labels(self) -> Sequence['outputs.GetNodePoolInitialNodeLabelResult']:
        """
        A list of key/value pairs to add to nodes after they join the Kubernetes cluster.
        """
        return pulumi.get(self, "initial_node_labels")

    @property
    @pulumi.getter(name="kubernetesVersion")
    def kubernetes_version(self) -> str:
        """
        The version of Kubernetes this node is running.
        """
        return pulumi.get(self, "kubernetes_version")

    @property
    @pulumi.getter(name="lifecycleDetails")
    def lifecycle_details(self) -> str:
        """
        Details about the state of the node.
        """
        return pulumi.get(self, "lifecycle_details")

    @property
    @pulumi.getter
    def name(self) -> str:
        """
        The name of the node.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="nodeConfigDetails")
    def node_config_details(self) -> Sequence['outputs.GetNodePoolNodeConfigDetailResult']:
        """
        The configuration of nodes in the node pool.
        """
        return pulumi.get(self, "node_config_details")

    @property
    @pulumi.getter(name="nodeEvictionNodePoolSettings")
    def node_eviction_node_pool_settings(self) -> Sequence['outputs.GetNodePoolNodeEvictionNodePoolSettingResult']:
        """
        Node Eviction Details configuration
        """
        return pulumi.get(self, "node_eviction_node_pool_settings")

    @property
    @pulumi.getter(name="nodeImageId")
    def node_image_id(self) -> str:
        """
        Deprecated. see `nodeSource`. The OCID of the image running on the nodes in the node pool.
        """
        return pulumi.get(self, "node_image_id")

    @property
    @pulumi.getter(name="nodeImageName")
    def node_image_name(self) -> str:
        """
        Deprecated. see `nodeSource`. The name of the image running on the nodes in the node pool.
        """
        return pulumi.get(self, "node_image_name")

    @property
    @pulumi.getter(name="nodeMetadata")
    def node_metadata(self) -> Mapping[str, Any]:
        """
        A list of key/value pairs to add to each underlying Oracle Cloud Infrastructure instance in the node pool on launch.
        """
        return pulumi.get(self, "node_metadata")

    @property
    @pulumi.getter(name="nodePoolCyclingDetails")
    def node_pool_cycling_details(self) -> Sequence['outputs.GetNodePoolNodePoolCyclingDetailResult']:
        """
        Node Pool Cycling Details
        """
        return pulumi.get(self, "node_pool_cycling_details")

    @property
    @pulumi.getter(name="nodePoolId")
    def node_pool_id(self) -> str:
        """
        The OCID of the node pool to which this node belongs.
        """
        return pulumi.get(self, "node_pool_id")

    @property
    @pulumi.getter(name="nodeShape")
    def node_shape(self) -> str:
        """
        The name of the node shape of the nodes in the node pool.
        """
        return pulumi.get(self, "node_shape")

    @property
    @pulumi.getter(name="nodeShapeConfigs")
    def node_shape_configs(self) -> Sequence['outputs.GetNodePoolNodeShapeConfigResult']:
        """
        The shape configuration of the nodes.
        """
        return pulumi.get(self, "node_shape_configs")

    @property
    @pulumi.getter(name="nodeSourceDetails")
    def node_source_details(self) -> Sequence['outputs.GetNodePoolNodeSourceDetailResult']:
        """
        Source running on the nodes in the node pool.
        """
        return pulumi.get(self, "node_source_details")

    @property
    @pulumi.getter(name="nodeSources")
    def node_sources(self) -> Sequence['outputs.GetNodePoolNodeSourceResult']:
        """
        Deprecated. see `nodeSourceDetails`. Source running on the nodes in the node pool.
        """
        return pulumi.get(self, "node_sources")

    @property
    @pulumi.getter
    def nodes(self) -> Sequence['outputs.GetNodePoolNodeResult']:
        """
        The nodes in the node pool.
        """
        return pulumi.get(self, "nodes")

    @property
    @pulumi.getter(name="quantityPerSubnet")
    def quantity_per_subnet(self) -> int:
        """
        The number of nodes in each subnet.
        """
        return pulumi.get(self, "quantity_per_subnet")

    @property
    @pulumi.getter(name="sshPublicKey")
    def ssh_public_key(self) -> str:
        """
        The SSH public key on each node in the node pool on launch.
        """
        return pulumi.get(self, "ssh_public_key")

    @property
    @pulumi.getter
    def state(self) -> str:
        """
        The state of the nodepool.
        """
        return pulumi.get(self, "state")

    @property
    @pulumi.getter(name="subnetIds")
    def subnet_ids(self) -> Sequence[str]:
        """
        The OCIDs of the subnets in which to place nodes for this node pool.
        """
        return pulumi.get(self, "subnet_ids")


class AwaitableGetNodePoolResult(GetNodePoolResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetNodePoolResult(
            cluster_id=self.cluster_id,
            compartment_id=self.compartment_id,
            defined_tags=self.defined_tags,
            freeform_tags=self.freeform_tags,
            id=self.id,
            initial_node_labels=self.initial_node_labels,
            kubernetes_version=self.kubernetes_version,
            lifecycle_details=self.lifecycle_details,
            name=self.name,
            node_config_details=self.node_config_details,
            node_eviction_node_pool_settings=self.node_eviction_node_pool_settings,
            node_image_id=self.node_image_id,
            node_image_name=self.node_image_name,
            node_metadata=self.node_metadata,
            node_pool_cycling_details=self.node_pool_cycling_details,
            node_pool_id=self.node_pool_id,
            node_shape=self.node_shape,
            node_shape_configs=self.node_shape_configs,
            node_source_details=self.node_source_details,
            node_sources=self.node_sources,
            nodes=self.nodes,
            quantity_per_subnet=self.quantity_per_subnet,
            ssh_public_key=self.ssh_public_key,
            state=self.state,
            subnet_ids=self.subnet_ids)


def get_node_pool(node_pool_id: Optional[str] = None,
                  opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetNodePoolResult:
    """
    This data source provides details about a specific Node Pool resource in Oracle Cloud Infrastructure Container Engine service.

    Get the details of a node pool.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_node_pool = oci.ContainerEngine.get_node_pool(node_pool_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str node_pool_id: The OCID of the node pool.
    """
    __args__ = dict()
    __args__['nodePoolId'] = node_pool_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('oci:ContainerEngine/getNodePool:getNodePool', __args__, opts=opts, typ=GetNodePoolResult).value

    return AwaitableGetNodePoolResult(
        cluster_id=__ret__.cluster_id,
        compartment_id=__ret__.compartment_id,
        defined_tags=__ret__.defined_tags,
        freeform_tags=__ret__.freeform_tags,
        id=__ret__.id,
        initial_node_labels=__ret__.initial_node_labels,
        kubernetes_version=__ret__.kubernetes_version,
        lifecycle_details=__ret__.lifecycle_details,
        name=__ret__.name,
        node_config_details=__ret__.node_config_details,
        node_eviction_node_pool_settings=__ret__.node_eviction_node_pool_settings,
        node_image_id=__ret__.node_image_id,
        node_image_name=__ret__.node_image_name,
        node_metadata=__ret__.node_metadata,
        node_pool_cycling_details=__ret__.node_pool_cycling_details,
        node_pool_id=__ret__.node_pool_id,
        node_shape=__ret__.node_shape,
        node_shape_configs=__ret__.node_shape_configs,
        node_source_details=__ret__.node_source_details,
        node_sources=__ret__.node_sources,
        nodes=__ret__.nodes,
        quantity_per_subnet=__ret__.quantity_per_subnet,
        ssh_public_key=__ret__.ssh_public_key,
        state=__ret__.state,
        subnet_ids=__ret__.subnet_ids)


@_utilities.lift_output_func(get_node_pool)
def get_node_pool_output(node_pool_id: Optional[pulumi.Input[str]] = None,
                         opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetNodePoolResult]:
    """
    This data source provides details about a specific Node Pool resource in Oracle Cloud Infrastructure Container Engine service.

    Get the details of a node pool.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_node_pool = oci.ContainerEngine.get_node_pool(node_pool_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str node_pool_id: The OCID of the node pool.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetLogsResult',
    'AwaitableGetLogsResult',
    'get_logs',
    'get_logs_output',
]

@pulumi.output_type
class GetLogsResult:
    """
    A collection of values returned by getLogs.
    """
    def __init__(__self__, display_name=None, filters=None, id=None, log_group_id=None, log_type=None, logs=None, source_resource=None, source_service=None, state=None):
        if display_name and not isinstance(display_name, str):
            raise TypeError("Expected argument 'display_name' to be a str")
        pulumi.set(__self__, "display_name", display_name)
        if filters and not isinstance(filters, list):
            raise TypeError("Expected argument 'filters' to be a list")
        pulumi.set(__self__, "filters", filters)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if log_group_id and not isinstance(log_group_id, str):
            raise TypeError("Expected argument 'log_group_id' to be a str")
        pulumi.set(__self__, "log_group_id", log_group_id)
        if log_type and not isinstance(log_type, str):
            raise TypeError("Expected argument 'log_type' to be a str")
        pulumi.set(__self__, "log_type", log_type)
        if logs and not isinstance(logs, list):
            raise TypeError("Expected argument 'logs' to be a list")
        pulumi.set(__self__, "logs", logs)
        if source_resource and not isinstance(source_resource, str):
            raise TypeError("Expected argument 'source_resource' to be a str")
        pulumi.set(__self__, "source_resource", source_resource)
        if source_service and not isinstance(source_service, str):
            raise TypeError("Expected argument 'source_service' to be a str")
        pulumi.set(__self__, "source_service", source_service)
        if state and not isinstance(state, str):
            raise TypeError("Expected argument 'state' to be a str")
        pulumi.set(__self__, "state", state)

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> Optional[str]:
        """
        The user-friendly display name. This must be unique within the enclosing resource, and it's changeable. Avoid entering confidential information.
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter
    def filters(self) -> Optional[Sequence['outputs.GetLogsFilterResult']]:
        return pulumi.get(self, "filters")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="logGroupId")
    def log_group_id(self) -> str:
        """
        Log group OCID.
        """
        return pulumi.get(self, "log_group_id")

    @property
    @pulumi.getter(name="logType")
    def log_type(self) -> Optional[str]:
        """
        The logType that the log object is for, whether custom or service.
        """
        return pulumi.get(self, "log_type")

    @property
    @pulumi.getter
    def logs(self) -> Sequence['outputs.GetLogsLogResult']:
        """
        The list of logs.
        """
        return pulumi.get(self, "logs")

    @property
    @pulumi.getter(name="sourceResource")
    def source_resource(self) -> Optional[str]:
        return pulumi.get(self, "source_resource")

    @property
    @pulumi.getter(name="sourceService")
    def source_service(self) -> Optional[str]:
        return pulumi.get(self, "source_service")

    @property
    @pulumi.getter
    def state(self) -> Optional[str]:
        """
        The pipeline state.
        """
        return pulumi.get(self, "state")


class AwaitableGetLogsResult(GetLogsResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetLogsResult(
            display_name=self.display_name,
            filters=self.filters,
            id=self.id,
            log_group_id=self.log_group_id,
            log_type=self.log_type,
            logs=self.logs,
            source_resource=self.source_resource,
            source_service=self.source_service,
            state=self.state)


def get_logs(display_name: Optional[str] = None,
             filters: Optional[Sequence[pulumi.InputType['GetLogsFilterArgs']]] = None,
             log_group_id: Optional[str] = None,
             log_type: Optional[str] = None,
             source_resource: Optional[str] = None,
             source_service: Optional[str] = None,
             state: Optional[str] = None,
             opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetLogsResult:
    """
    This data source provides the list of Logs in Oracle Cloud Infrastructure Logging service.

    Lists the specified log group's log objects.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_logs = oci.Logging.get_logs(log_group_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        log_type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        source_resource=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        source_service=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        state=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str display_name: Resource name.
    :param str log_group_id: OCID of a log group to work with.
    :param str log_type: The logType that the log object is for, whether custom or service.
    :param str source_resource: Log object resource, which is a field of LogSummary.Configuration.Source.
    :param str source_service: Service that created the log object, which is a field of LogSummary.Configuration.Source.
    :param str state: Lifecycle state of the log object
    """
    __args__ = dict()
    __args__['displayName'] = display_name
    __args__['filters'] = filters
    __args__['logGroupId'] = log_group_id
    __args__['logType'] = log_type
    __args__['sourceResource'] = source_resource
    __args__['sourceService'] = source_service
    __args__['state'] = state
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('oci:Logging/getLogs:getLogs', __args__, opts=opts, typ=GetLogsResult).value

    return AwaitableGetLogsResult(
        display_name=__ret__.display_name,
        filters=__ret__.filters,
        id=__ret__.id,
        log_group_id=__ret__.log_group_id,
        log_type=__ret__.log_type,
        logs=__ret__.logs,
        source_resource=__ret__.source_resource,
        source_service=__ret__.source_service,
        state=__ret__.state)


@_utilities.lift_output_func(get_logs)
def get_logs_output(display_name: Optional[pulumi.Input[Optional[str]]] = None,
                    filters: Optional[pulumi.Input[Optional[Sequence[pulumi.InputType['GetLogsFilterArgs']]]]] = None,
                    log_group_id: Optional[pulumi.Input[str]] = None,
                    log_type: Optional[pulumi.Input[Optional[str]]] = None,
                    source_resource: Optional[pulumi.Input[Optional[str]]] = None,
                    source_service: Optional[pulumi.Input[Optional[str]]] = None,
                    state: Optional[pulumi.Input[Optional[str]]] = None,
                    opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetLogsResult]:
    """
    This data source provides the list of Logs in Oracle Cloud Infrastructure Logging service.

    Lists the specified log group's log objects.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_logs = oci.Logging.get_logs(log_group_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        log_type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        source_resource=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        source_service=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        state=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str display_name: Resource name.
    :param str log_group_id: OCID of a log group to work with.
    :param str log_type: The logType that the log object is for, whether custom or service.
    :param str source_resource: Log object resource, which is a field of LogSummary.Configuration.Source.
    :param str source_service: Service that created the log object, which is a field of LogSummary.Configuration.Source.
    :param str state: Lifecycle state of the log object
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['AutonomousDatabaseBackupArgs', 'AutonomousDatabaseBackup']

@pulumi.input_type
class AutonomousDatabaseBackupArgs:
    def __init__(__self__, *,
                 autonomous_database_id: pulumi.Input[str],
                 backup_destination_details: Optional[pulumi.Input['AutonomousDatabaseBackupBackupDestinationDetailsArgs']] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 is_long_term_backup: Optional[pulumi.Input[bool]] = None,
                 retention_period_in_days: Optional[pulumi.Input[int]] = None):
        """
        The set of arguments for constructing a AutonomousDatabaseBackup resource.
        :param pulumi.Input[str] autonomous_database_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the Autonomous Database backup.
        :param pulumi.Input['AutonomousDatabaseBackupBackupDestinationDetailsArgs'] backup_destination_details: Backup destination details
        :param pulumi.Input[str] display_name: The user-friendly name for the backup. The name does not have to be unique.
        :param pulumi.Input[bool] is_long_term_backup: Indicates whether the backup is long-term
        :param pulumi.Input[int] retention_period_in_days: (Updatable) Retention period, in days, for long-term backups
               
               
               ** IMPORTANT **
               Any change to a property that does not support update will force the destruction and recreation of the resource with the new property values
        """
        pulumi.set(__self__, "autonomous_database_id", autonomous_database_id)
        if backup_destination_details is not None:
            pulumi.set(__self__, "backup_destination_details", backup_destination_details)
        if display_name is not None:
            pulumi.set(__self__, "display_name", display_name)
        if is_long_term_backup is not None:
            pulumi.set(__self__, "is_long_term_backup", is_long_term_backup)
        if retention_period_in_days is not None:
            pulumi.set(__self__, "retention_period_in_days", retention_period_in_days)

    @property
    @pulumi.getter(name="autonomousDatabaseId")
    def autonomous_database_id(self) -> pulumi.Input[str]:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the Autonomous Database backup.
        """
        return pulumi.get(self, "autonomous_database_id")

    @autonomous_database_id.setter
    def autonomous_database_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "autonomous_database_id", value)

    @property
    @pulumi.getter(name="backupDestinationDetails")
    def backup_destination_details(self) -> Optional[pulumi.Input['AutonomousDatabaseBackupBackupDestinationDetailsArgs']]:
        """
        Backup destination details
        """
        return pulumi.get(self, "backup_destination_details")

    @backup_destination_details.setter
    def backup_destination_details(self, value: Optional[pulumi.Input['AutonomousDatabaseBackupBackupDestinationDetailsArgs']]):
        pulumi.set(self, "backup_destination_details", value)

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> Optional[pulumi.Input[str]]:
        """
        The user-friendly name for the backup. The name does not have to be unique.
        """
        return pulumi.get(self, "display_name")

    @display_name.setter
    def display_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "display_name", value)

    @property
    @pulumi.getter(name="isLongTermBackup")
    def is_long_term_backup(self) -> Optional[pulumi.Input[bool]]:
        """
        Indicates whether the backup is long-term
        """
        return pulumi.get(self, "is_long_term_backup")

    @is_long_term_backup.setter
    def is_long_term_backup(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "is_long_term_backup", value)

    @property
    @pulumi.getter(name="retentionPeriodInDays")
    def retention_period_in_days(self) -> Optional[pulumi.Input[int]]:
        """
        (Updatable) Retention period, in days, for long-term backups


        ** IMPORTANT **
        Any change to a property that does not support update will force the destruction and recreation of the resource with the new property values
        """
        return pulumi.get(self, "retention_period_in_days")

    @retention_period_in_days.setter
    def retention_period_in_days(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "retention_period_in_days", value)


@pulumi.input_type
class _AutonomousDatabaseBackupState:
    def __init__(__self__, *,
                 autonomous_database_id: Optional[pulumi.Input[str]] = None,
                 backup_destination_details: Optional[pulumi.Input['AutonomousDatabaseBackupBackupDestinationDetailsArgs']] = None,
                 compartment_id: Optional[pulumi.Input[str]] = None,
                 database_size_in_tbs: Optional[pulumi.Input[float]] = None,
                 db_version: Optional[pulumi.Input[str]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 is_automatic: Optional[pulumi.Input[bool]] = None,
                 is_long_term_backup: Optional[pulumi.Input[bool]] = None,
                 is_restorable: Optional[pulumi.Input[bool]] = None,
                 key_store_id: Optional[pulumi.Input[str]] = None,
                 key_store_wallet_name: Optional[pulumi.Input[str]] = None,
                 kms_key_id: Optional[pulumi.Input[str]] = None,
                 kms_key_version_id: Optional[pulumi.Input[str]] = None,
                 lifecycle_details: Optional[pulumi.Input[str]] = None,
                 retention_period_in_days: Optional[pulumi.Input[int]] = None,
                 size_in_tbs: Optional[pulumi.Input[float]] = None,
                 state: Optional[pulumi.Input[str]] = None,
                 time_available_till: Optional[pulumi.Input[str]] = None,
                 time_ended: Optional[pulumi.Input[str]] = None,
                 time_started: Optional[pulumi.Input[str]] = None,
                 type: Optional[pulumi.Input[str]] = None,
                 vault_id: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering AutonomousDatabaseBackup resources.
        :param pulumi.Input[str] autonomous_database_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the Autonomous Database backup.
        :param pulumi.Input['AutonomousDatabaseBackupBackupDestinationDetailsArgs'] backup_destination_details: Backup destination details
        :param pulumi.Input[str] compartment_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the compartment.
        :param pulumi.Input[float] database_size_in_tbs: The size of the database in terabytes at the time the backup was taken.
        :param pulumi.Input[str] db_version: A valid Oracle Database version for Autonomous Database.
        :param pulumi.Input[str] display_name: The user-friendly name for the backup. The name does not have to be unique.
        :param pulumi.Input[bool] is_automatic: Indicates whether the backup is user-initiated or automatic.
        :param pulumi.Input[bool] is_long_term_backup: Indicates whether the backup is long-term
        :param pulumi.Input[bool] is_restorable: Indicates whether the backup can be used to restore the associated Autonomous Database.
        :param pulumi.Input[str] key_store_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the key store.
        :param pulumi.Input[str] key_store_wallet_name: The wallet name for Oracle Key Vault.
        :param pulumi.Input[str] kms_key_id: The OCID of the key container that is used as the master encryption key in database transparent data encryption (TDE) operations.
        :param pulumi.Input[str] kms_key_version_id: The OCID of the key container version that is used in database transparent data encryption (TDE) operations KMS Key can have multiple key versions. If none is specified, the current key version (latest) of the Key Id is used for the operation.
        :param pulumi.Input[str] lifecycle_details: Additional information about the current lifecycle state.
        :param pulumi.Input[int] retention_period_in_days: (Updatable) Retention period, in days, for long-term backups
               
               
               ** IMPORTANT **
               Any change to a property that does not support update will force the destruction and recreation of the resource with the new property values
        :param pulumi.Input[float] size_in_tbs: The backup size in terrabytes (TB).
        :param pulumi.Input[str] state: The current state of the backup.
        :param pulumi.Input[str] time_available_till: Timestamp until when the backup will be available
        :param pulumi.Input[str] time_ended: The date and time the backup completed.
        :param pulumi.Input[str] time_started: The date and time the backup started.
        :param pulumi.Input[str] type: The type of backup.
        :param pulumi.Input[str] vault_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the Oracle Cloud Infrastructure [vault](https://docs.cloud.oracle.com/iaas/Content/KeyManagement/Concepts/keyoverview.htm#concepts).
        """
        if autonomous_database_id is not None:
            pulumi.set(__self__, "autonomous_database_id", autonomous_database_id)
        if backup_destination_details is not None:
            pulumi.set(__self__, "backup_destination_details", backup_destination_details)
        if compartment_id is not None:
            pulumi.set(__self__, "compartment_id", compartment_id)
        if database_size_in_tbs is not None:
            pulumi.set(__self__, "database_size_in_tbs", database_size_in_tbs)
        if db_version is not None:
            pulumi.set(__self__, "db_version", db_version)
        if display_name is not None:
            pulumi.set(__self__, "display_name", display_name)
        if is_automatic is not None:
            pulumi.set(__self__, "is_automatic", is_automatic)
        if is_long_term_backup is not None:
            pulumi.set(__self__, "is_long_term_backup", is_long_term_backup)
        if is_restorable is not None:
            pulumi.set(__self__, "is_restorable", is_restorable)
        if key_store_id is not None:
            pulumi.set(__self__, "key_store_id", key_store_id)
        if key_store_wallet_name is not None:
            pulumi.set(__self__, "key_store_wallet_name", key_store_wallet_name)
        if kms_key_id is not None:
            pulumi.set(__self__, "kms_key_id", kms_key_id)
        if kms_key_version_id is not None:
            pulumi.set(__self__, "kms_key_version_id", kms_key_version_id)
        if lifecycle_details is not None:
            pulumi.set(__self__, "lifecycle_details", lifecycle_details)
        if retention_period_in_days is not None:
            pulumi.set(__self__, "retention_period_in_days", retention_period_in_days)
        if size_in_tbs is not None:
            pulumi.set(__self__, "size_in_tbs", size_in_tbs)
        if state is not None:
            pulumi.set(__self__, "state", state)
        if time_available_till is not None:
            pulumi.set(__self__, "time_available_till", time_available_till)
        if time_ended is not None:
            pulumi.set(__self__, "time_ended", time_ended)
        if time_started is not None:
            pulumi.set(__self__, "time_started", time_started)
        if type is not None:
            pulumi.set(__self__, "type", type)
        if vault_id is not None:
            pulumi.set(__self__, "vault_id", vault_id)

    @property
    @pulumi.getter(name="autonomousDatabaseId")
    def autonomous_database_id(self) -> Optional[pulumi.Input[str]]:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the Autonomous Database backup.
        """
        return pulumi.get(self, "autonomous_database_id")

    @autonomous_database_id.setter
    def autonomous_database_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "autonomous_database_id", value)

    @property
    @pulumi.getter(name="backupDestinationDetails")
    def backup_destination_details(self) -> Optional[pulumi.Input['AutonomousDatabaseBackupBackupDestinationDetailsArgs']]:
        """
        Backup destination details
        """
        return pulumi.get(self, "backup_destination_details")

    @backup_destination_details.setter
    def backup_destination_details(self, value: Optional[pulumi.Input['AutonomousDatabaseBackupBackupDestinationDetailsArgs']]):
        pulumi.set(self, "backup_destination_details", value)

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> Optional[pulumi.Input[str]]:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the compartment.
        """
        return pulumi.get(self, "compartment_id")

    @compartment_id.setter
    def compartment_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "compartment_id", value)

    @property
    @pulumi.getter(name="databaseSizeInTbs")
    def database_size_in_tbs(self) -> Optional[pulumi.Input[float]]:
        """
        The size of the database in terabytes at the time the backup was taken.
        """
        return pulumi.get(self, "database_size_in_tbs")

    @database_size_in_tbs.setter
    def database_size_in_tbs(self, value: Optional[pulumi.Input[float]]):
        pulumi.set(self, "database_size_in_tbs", value)

    @property
    @pulumi.getter(name="dbVersion")
    def db_version(self) -> Optional[pulumi.Input[str]]:
        """
        A valid Oracle Database version for Autonomous Database.
        """
        return pulumi.get(self, "db_version")

    @db_version.setter
    def db_version(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "db_version", value)

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> Optional[pulumi.Input[str]]:
        """
        The user-friendly name for the backup. The name does not have to be unique.
        """
        return pulumi.get(self, "display_name")

    @display_name.setter
    def display_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "display_name", value)

    @property
    @pulumi.getter(name="isAutomatic")
    def is_automatic(self) -> Optional[pulumi.Input[bool]]:
        """
        Indicates whether the backup is user-initiated or automatic.
        """
        return pulumi.get(self, "is_automatic")

    @is_automatic.setter
    def is_automatic(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "is_automatic", value)

    @property
    @pulumi.getter(name="isLongTermBackup")
    def is_long_term_backup(self) -> Optional[pulumi.Input[bool]]:
        """
        Indicates whether the backup is long-term
        """
        return pulumi.get(self, "is_long_term_backup")

    @is_long_term_backup.setter
    def is_long_term_backup(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "is_long_term_backup", value)

    @property
    @pulumi.getter(name="isRestorable")
    def is_restorable(self) -> Optional[pulumi.Input[bool]]:
        """
        Indicates whether the backup can be used to restore the associated Autonomous Database.
        """
        return pulumi.get(self, "is_restorable")

    @is_restorable.setter
    def is_restorable(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "is_restorable", value)

    @property
    @pulumi.getter(name="keyStoreId")
    def key_store_id(self) -> Optional[pulumi.Input[str]]:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the key store.
        """
        return pulumi.get(self, "key_store_id")

    @key_store_id.setter
    def key_store_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "key_store_id", value)

    @property
    @pulumi.getter(name="keyStoreWalletName")
    def key_store_wallet_name(self) -> Optional[pulumi.Input[str]]:
        """
        The wallet name for Oracle Key Vault.
        """
        return pulumi.get(self, "key_store_wallet_name")

    @key_store_wallet_name.setter
    def key_store_wallet_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "key_store_wallet_name", value)

    @property
    @pulumi.getter(name="kmsKeyId")
    def kms_key_id(self) -> Optional[pulumi.Input[str]]:
        """
        The OCID of the key container that is used as the master encryption key in database transparent data encryption (TDE) operations.
        """
        return pulumi.get(self, "kms_key_id")

    @kms_key_id.setter
    def kms_key_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "kms_key_id", value)

    @property
    @pulumi.getter(name="kmsKeyVersionId")
    def kms_key_version_id(self) -> Optional[pulumi.Input[str]]:
        """
        The OCID of the key container version that is used in database transparent data encryption (TDE) operations KMS Key can have multiple key versions. If none is specified, the current key version (latest) of the Key Id is used for the operation.
        """
        return pulumi.get(self, "kms_key_version_id")

    @kms_key_version_id.setter
    def kms_key_version_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "kms_key_version_id", value)

    @property
    @pulumi.getter(name="lifecycleDetails")
    def lifecycle_details(self) -> Optional[pulumi.Input[str]]:
        """
        Additional information about the current lifecycle state.
        """
        return pulumi.get(self, "lifecycle_details")

    @lifecycle_details.setter
    def lifecycle_details(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "lifecycle_details", value)

    @property
    @pulumi.getter(name="retentionPeriodInDays")
    def retention_period_in_days(self) -> Optional[pulumi.Input[int]]:
        """
        (Updatable) Retention period, in days, for long-term backups


        ** IMPORTANT **
        Any change to a property that does not support update will force the destruction and recreation of the resource with the new property values
        """
        return pulumi.get(self, "retention_period_in_days")

    @retention_period_in_days.setter
    def retention_period_in_days(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "retention_period_in_days", value)

    @property
    @pulumi.getter(name="sizeInTbs")
    def size_in_tbs(self) -> Optional[pulumi.Input[float]]:
        """
        The backup size in terrabytes (TB).
        """
        return pulumi.get(self, "size_in_tbs")

    @size_in_tbs.setter
    def size_in_tbs(self, value: Optional[pulumi.Input[float]]):
        pulumi.set(self, "size_in_tbs", value)

    @property
    @pulumi.getter
    def state(self) -> Optional[pulumi.Input[str]]:
        """
        The current state of the backup.
        """
        return pulumi.get(self, "state")

    @state.setter
    def state(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "state", value)

    @property
    @pulumi.getter(name="timeAvailableTill")
    def time_available_till(self) -> Optional[pulumi.Input[str]]:
        """
        Timestamp until when the backup will be available
        """
        return pulumi.get(self, "time_available_till")

    @time_available_till.setter
    def time_available_till(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "time_available_till", value)

    @property
    @pulumi.getter(name="timeEnded")
    def time_ended(self) -> Optional[pulumi.Input[str]]:
        """
        The date and time the backup completed.
        """
        return pulumi.get(self, "time_ended")

    @time_ended.setter
    def time_ended(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "time_ended", value)

    @property
    @pulumi.getter(name="timeStarted")
    def time_started(self) -> Optional[pulumi.Input[str]]:
        """
        The date and time the backup started.
        """
        return pulumi.get(self, "time_started")

    @time_started.setter
    def time_started(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "time_started", value)

    @property
    @pulumi.getter
    def type(self) -> Optional[pulumi.Input[str]]:
        """
        The type of backup.
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "type", value)

    @property
    @pulumi.getter(name="vaultId")
    def vault_id(self) -> Optional[pulumi.Input[str]]:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the Oracle Cloud Infrastructure [vault](https://docs.cloud.oracle.com/iaas/Content/KeyManagement/Concepts/keyoverview.htm#concepts).
        """
        return pulumi.get(self, "vault_id")

    @vault_id.setter
    def vault_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "vault_id", value)


class AutonomousDatabaseBackup(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 autonomous_database_id: Optional[pulumi.Input[str]] = None,
                 backup_destination_details: Optional[pulumi.Input[pulumi.InputType['AutonomousDatabaseBackupBackupDestinationDetailsArgs']]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 is_long_term_backup: Optional[pulumi.Input[bool]] = None,
                 retention_period_in_days: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        """
        This resource provides the Autonomous Database Backup resource in Oracle Cloud Infrastructure Database service.

        Creates a new Autonomous Database backup for the specified database based on the provided request parameters.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_oci as oci

        test_autonomous_database_backup = oci.database.AutonomousDatabaseBackup("testAutonomousDatabaseBackup",
            autonomous_database_id=oci_database_autonomous_database["test_autonomous_database"]["id"],
            display_name=var["autonomous_database_backup_display_name"],
            is_long_term_backup=var["autonomous_database_backup_is_long_term_backup"],
            retention_period_in_days=var["autonomous_database_backup_retention_period_in_days"])
        ```

        ## Import

        AutonomousDatabaseBackups can be imported using the `id`, e.g.

        ```sh
         $ pulumi import oci:Database/autonomousDatabaseBackup:AutonomousDatabaseBackup test_autonomous_database_backup "id"
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] autonomous_database_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the Autonomous Database backup.
        :param pulumi.Input[pulumi.InputType['AutonomousDatabaseBackupBackupDestinationDetailsArgs']] backup_destination_details: Backup destination details
        :param pulumi.Input[str] display_name: The user-friendly name for the backup. The name does not have to be unique.
        :param pulumi.Input[bool] is_long_term_backup: Indicates whether the backup is long-term
        :param pulumi.Input[int] retention_period_in_days: (Updatable) Retention period, in days, for long-term backups
               
               
               ** IMPORTANT **
               Any change to a property that does not support update will force the destruction and recreation of the resource with the new property values
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: AutonomousDatabaseBackupArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        This resource provides the Autonomous Database Backup resource in Oracle Cloud Infrastructure Database service.

        Creates a new Autonomous Database backup for the specified database based on the provided request parameters.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_oci as oci

        test_autonomous_database_backup = oci.database.AutonomousDatabaseBackup("testAutonomousDatabaseBackup",
            autonomous_database_id=oci_database_autonomous_database["test_autonomous_database"]["id"],
            display_name=var["autonomous_database_backup_display_name"],
            is_long_term_backup=var["autonomous_database_backup_is_long_term_backup"],
            retention_period_in_days=var["autonomous_database_backup_retention_period_in_days"])
        ```

        ## Import

        AutonomousDatabaseBackups can be imported using the `id`, e.g.

        ```sh
         $ pulumi import oci:Database/autonomousDatabaseBackup:AutonomousDatabaseBackup test_autonomous_database_backup "id"
        ```

        :param str resource_name: The name of the resource.
        :param AutonomousDatabaseBackupArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(AutonomousDatabaseBackupArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 autonomous_database_id: Optional[pulumi.Input[str]] = None,
                 backup_destination_details: Optional[pulumi.Input[pulumi.InputType['AutonomousDatabaseBackupBackupDestinationDetailsArgs']]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 is_long_term_backup: Optional[pulumi.Input[bool]] = None,
                 retention_period_in_days: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = AutonomousDatabaseBackupArgs.__new__(AutonomousDatabaseBackupArgs)

            if autonomous_database_id is None and not opts.urn:
                raise TypeError("Missing required property 'autonomous_database_id'")
            __props__.__dict__["autonomous_database_id"] = autonomous_database_id
            __props__.__dict__["backup_destination_details"] = backup_destination_details
            __props__.__dict__["display_name"] = display_name
            __props__.__dict__["is_long_term_backup"] = is_long_term_backup
            __props__.__dict__["retention_period_in_days"] = retention_period_in_days
            __props__.__dict__["compartment_id"] = None
            __props__.__dict__["database_size_in_tbs"] = None
            __props__.__dict__["db_version"] = None
            __props__.__dict__["is_automatic"] = None
            __props__.__dict__["is_restorable"] = None
            __props__.__dict__["key_store_id"] = None
            __props__.__dict__["key_store_wallet_name"] = None
            __props__.__dict__["kms_key_id"] = None
            __props__.__dict__["kms_key_version_id"] = None
            __props__.__dict__["lifecycle_details"] = None
            __props__.__dict__["size_in_tbs"] = None
            __props__.__dict__["state"] = None
            __props__.__dict__["time_available_till"] = None
            __props__.__dict__["time_ended"] = None
            __props__.__dict__["time_started"] = None
            __props__.__dict__["type"] = None
            __props__.__dict__["vault_id"] = None
        super(AutonomousDatabaseBackup, __self__).__init__(
            'oci:Database/autonomousDatabaseBackup:AutonomousDatabaseBackup',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            autonomous_database_id: Optional[pulumi.Input[str]] = None,
            backup_destination_details: Optional[pulumi.Input[pulumi.InputType['AutonomousDatabaseBackupBackupDestinationDetailsArgs']]] = None,
            compartment_id: Optional[pulumi.Input[str]] = None,
            database_size_in_tbs: Optional[pulumi.Input[float]] = None,
            db_version: Optional[pulumi.Input[str]] = None,
            display_name: Optional[pulumi.Input[str]] = None,
            is_automatic: Optional[pulumi.Input[bool]] = None,
            is_long_term_backup: Optional[pulumi.Input[bool]] = None,
            is_restorable: Optional[pulumi.Input[bool]] = None,
            key_store_id: Optional[pulumi.Input[str]] = None,
            key_store_wallet_name: Optional[pulumi.Input[str]] = None,
            kms_key_id: Optional[pulumi.Input[str]] = None,
            kms_key_version_id: Optional[pulumi.Input[str]] = None,
            lifecycle_details: Optional[pulumi.Input[str]] = None,
            retention_period_in_days: Optional[pulumi.Input[int]] = None,
            size_in_tbs: Optional[pulumi.Input[float]] = None,
            state: Optional[pulumi.Input[str]] = None,
            time_available_till: Optional[pulumi.Input[str]] = None,
            time_ended: Optional[pulumi.Input[str]] = None,
            time_started: Optional[pulumi.Input[str]] = None,
            type: Optional[pulumi.Input[str]] = None,
            vault_id: Optional[pulumi.Input[str]] = None) -> 'AutonomousDatabaseBackup':
        """
        Get an existing AutonomousDatabaseBackup resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] autonomous_database_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the Autonomous Database backup.
        :param pulumi.Input[pulumi.InputType['AutonomousDatabaseBackupBackupDestinationDetailsArgs']] backup_destination_details: Backup destination details
        :param pulumi.Input[str] compartment_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the compartment.
        :param pulumi.Input[float] database_size_in_tbs: The size of the database in terabytes at the time the backup was taken.
        :param pulumi.Input[str] db_version: A valid Oracle Database version for Autonomous Database.
        :param pulumi.Input[str] display_name: The user-friendly name for the backup. The name does not have to be unique.
        :param pulumi.Input[bool] is_automatic: Indicates whether the backup is user-initiated or automatic.
        :param pulumi.Input[bool] is_long_term_backup: Indicates whether the backup is long-term
        :param pulumi.Input[bool] is_restorable: Indicates whether the backup can be used to restore the associated Autonomous Database.
        :param pulumi.Input[str] key_store_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the key store.
        :param pulumi.Input[str] key_store_wallet_name: The wallet name for Oracle Key Vault.
        :param pulumi.Input[str] kms_key_id: The OCID of the key container that is used as the master encryption key in database transparent data encryption (TDE) operations.
        :param pulumi.Input[str] kms_key_version_id: The OCID of the key container version that is used in database transparent data encryption (TDE) operations KMS Key can have multiple key versions. If none is specified, the current key version (latest) of the Key Id is used for the operation.
        :param pulumi.Input[str] lifecycle_details: Additional information about the current lifecycle state.
        :param pulumi.Input[int] retention_period_in_days: (Updatable) Retention period, in days, for long-term backups
               
               
               ** IMPORTANT **
               Any change to a property that does not support update will force the destruction and recreation of the resource with the new property values
        :param pulumi.Input[float] size_in_tbs: The backup size in terrabytes (TB).
        :param pulumi.Input[str] state: The current state of the backup.
        :param pulumi.Input[str] time_available_till: Timestamp until when the backup will be available
        :param pulumi.Input[str] time_ended: The date and time the backup completed.
        :param pulumi.Input[str] time_started: The date and time the backup started.
        :param pulumi.Input[str] type: The type of backup.
        :param pulumi.Input[str] vault_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the Oracle Cloud Infrastructure [vault](https://docs.cloud.oracle.com/iaas/Content/KeyManagement/Concepts/keyoverview.htm#concepts).
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _AutonomousDatabaseBackupState.__new__(_AutonomousDatabaseBackupState)

        __props__.__dict__["autonomous_database_id"] = autonomous_database_id
        __props__.__dict__["backup_destination_details"] = backup_destination_details
        __props__.__dict__["compartment_id"] = compartment_id
        __props__.__dict__["database_size_in_tbs"] = database_size_in_tbs
        __props__.__dict__["db_version"] = db_version
        __props__.__dict__["display_name"] = display_name
        __props__.__dict__["is_automatic"] = is_automatic
        __props__.__dict__["is_long_term_backup"] = is_long_term_backup
        __props__.__dict__["is_restorable"] = is_restorable
        __props__.__dict__["key_store_id"] = key_store_id
        __props__.__dict__["key_store_wallet_name"] = key_store_wallet_name
        __props__.__dict__["kms_key_id"] = kms_key_id
        __props__.__dict__["kms_key_version_id"] = kms_key_version_id
        __props__.__dict__["lifecycle_details"] = lifecycle_details
        __props__.__dict__["retention_period_in_days"] = retention_period_in_days
        __props__.__dict__["size_in_tbs"] = size_in_tbs
        __props__.__dict__["state"] = state
        __props__.__dict__["time_available_till"] = time_available_till
        __props__.__dict__["time_ended"] = time_ended
        __props__.__dict__["time_started"] = time_started
        __props__.__dict__["type"] = type
        __props__.__dict__["vault_id"] = vault_id
        return AutonomousDatabaseBackup(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="autonomousDatabaseId")
    def autonomous_database_id(self) -> pulumi.Output[str]:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the Autonomous Database backup.
        """
        return pulumi.get(self, "autonomous_database_id")

    @property
    @pulumi.getter(name="backupDestinationDetails")
    def backup_destination_details(self) -> pulumi.Output['outputs.AutonomousDatabaseBackupBackupDestinationDetails']:
        """
        Backup destination details
        """
        return pulumi.get(self, "backup_destination_details")

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> pulumi.Output[str]:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the compartment.
        """
        return pulumi.get(self, "compartment_id")

    @property
    @pulumi.getter(name="databaseSizeInTbs")
    def database_size_in_tbs(self) -> pulumi.Output[float]:
        """
        The size of the database in terabytes at the time the backup was taken.
        """
        return pulumi.get(self, "database_size_in_tbs")

    @property
    @pulumi.getter(name="dbVersion")
    def db_version(self) -> pulumi.Output[str]:
        """
        A valid Oracle Database version for Autonomous Database.
        """
        return pulumi.get(self, "db_version")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> pulumi.Output[str]:
        """
        The user-friendly name for the backup. The name does not have to be unique.
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter(name="isAutomatic")
    def is_automatic(self) -> pulumi.Output[bool]:
        """
        Indicates whether the backup is user-initiated or automatic.
        """
        return pulumi.get(self, "is_automatic")

    @property
    @pulumi.getter(name="isLongTermBackup")
    def is_long_term_backup(self) -> pulumi.Output[bool]:
        """
        Indicates whether the backup is long-term
        """
        return pulumi.get(self, "is_long_term_backup")

    @property
    @pulumi.getter(name="isRestorable")
    def is_restorable(self) -> pulumi.Output[bool]:
        """
        Indicates whether the backup can be used to restore the associated Autonomous Database.
        """
        return pulumi.get(self, "is_restorable")

    @property
    @pulumi.getter(name="keyStoreId")
    def key_store_id(self) -> pulumi.Output[str]:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the key store.
        """
        return pulumi.get(self, "key_store_id")

    @property
    @pulumi.getter(name="keyStoreWalletName")
    def key_store_wallet_name(self) -> pulumi.Output[str]:
        """
        The wallet name for Oracle Key Vault.
        """
        return pulumi.get(self, "key_store_wallet_name")

    @property
    @pulumi.getter(name="kmsKeyId")
    def kms_key_id(self) -> pulumi.Output[str]:
        """
        The OCID of the key container that is used as the master encryption key in database transparent data encryption (TDE) operations.
        """
        return pulumi.get(self, "kms_key_id")

    @property
    @pulumi.getter(name="kmsKeyVersionId")
    def kms_key_version_id(self) -> pulumi.Output[str]:
        """
        The OCID of the key container version that is used in database transparent data encryption (TDE) operations KMS Key can have multiple key versions. If none is specified, the current key version (latest) of the Key Id is used for the operation.
        """
        return pulumi.get(self, "kms_key_version_id")

    @property
    @pulumi.getter(name="lifecycleDetails")
    def lifecycle_details(self) -> pulumi.Output[str]:
        """
        Additional information about the current lifecycle state.
        """
        return pulumi.get(self, "lifecycle_details")

    @property
    @pulumi.getter(name="retentionPeriodInDays")
    def retention_period_in_days(self) -> pulumi.Output[int]:
        """
        (Updatable) Retention period, in days, for long-term backups


        ** IMPORTANT **
        Any change to a property that does not support update will force the destruction and recreation of the resource with the new property values
        """
        return pulumi.get(self, "retention_period_in_days")

    @property
    @pulumi.getter(name="sizeInTbs")
    def size_in_tbs(self) -> pulumi.Output[float]:
        """
        The backup size in terrabytes (TB).
        """
        return pulumi.get(self, "size_in_tbs")

    @property
    @pulumi.getter
    def state(self) -> pulumi.Output[str]:
        """
        The current state of the backup.
        """
        return pulumi.get(self, "state")

    @property
    @pulumi.getter(name="timeAvailableTill")
    def time_available_till(self) -> pulumi.Output[str]:
        """
        Timestamp until when the backup will be available
        """
        return pulumi.get(self, "time_available_till")

    @property
    @pulumi.getter(name="timeEnded")
    def time_ended(self) -> pulumi.Output[str]:
        """
        The date and time the backup completed.
        """
        return pulumi.get(self, "time_ended")

    @property
    @pulumi.getter(name="timeStarted")
    def time_started(self) -> pulumi.Output[str]:
        """
        The date and time the backup started.
        """
        return pulumi.get(self, "time_started")

    @property
    @pulumi.getter
    def type(self) -> pulumi.Output[str]:
        """
        The type of backup.
        """
        return pulumi.get(self, "type")

    @property
    @pulumi.getter(name="vaultId")
    def vault_id(self) -> pulumi.Output[str]:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the Oracle Cloud Infrastructure [vault](https://docs.cloud.oracle.com/iaas/Content/KeyManagement/Concepts/keyoverview.htm#concepts).
        """
        return pulumi.get(self, "vault_id")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = [
    'GetRepositoryRefResult',
    'AwaitableGetRepositoryRefResult',
    'get_repository_ref',
    'get_repository_ref_output',
]

@pulumi.output_type
class GetRepositoryRefResult:
    """
    A collection of values returned by getRepositoryRef.
    """
    def __init__(__self__, commit_id=None, defined_tags=None, freeform_tags=None, full_ref_name=None, id=None, object_id=None, ref_name=None, ref_type=None, repository_id=None):
        if commit_id and not isinstance(commit_id, str):
            raise TypeError("Expected argument 'commit_id' to be a str")
        pulumi.set(__self__, "commit_id", commit_id)
        if defined_tags and not isinstance(defined_tags, dict):
            raise TypeError("Expected argument 'defined_tags' to be a dict")
        pulumi.set(__self__, "defined_tags", defined_tags)
        if freeform_tags and not isinstance(freeform_tags, dict):
            raise TypeError("Expected argument 'freeform_tags' to be a dict")
        pulumi.set(__self__, "freeform_tags", freeform_tags)
        if full_ref_name and not isinstance(full_ref_name, str):
            raise TypeError("Expected argument 'full_ref_name' to be a str")
        pulumi.set(__self__, "full_ref_name", full_ref_name)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if object_id and not isinstance(object_id, str):
            raise TypeError("Expected argument 'object_id' to be a str")
        pulumi.set(__self__, "object_id", object_id)
        if ref_name and not isinstance(ref_name, str):
            raise TypeError("Expected argument 'ref_name' to be a str")
        pulumi.set(__self__, "ref_name", ref_name)
        if ref_type and not isinstance(ref_type, str):
            raise TypeError("Expected argument 'ref_type' to be a str")
        pulumi.set(__self__, "ref_type", ref_type)
        if repository_id and not isinstance(repository_id, str):
            raise TypeError("Expected argument 'repository_id' to be a str")
        pulumi.set(__self__, "repository_id", repository_id)

    @property
    @pulumi.getter(name="commitId")
    def commit_id(self) -> str:
        """
        Commit ID pointed to by the new branch.
        """
        return pulumi.get(self, "commit_id")

    @property
    @pulumi.getter(name="definedTags")
    def defined_tags(self) -> Mapping[str, Any]:
        """
        Defined tags for this resource. Each key is predefined and scoped to a namespace. See [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm). Example: `{"foo-namespace.bar-key": "value"}`
        """
        return pulumi.get(self, "defined_tags")

    @property
    @pulumi.getter(name="freeformTags")
    def freeform_tags(self) -> Mapping[str, Any]:
        """
        Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only.  See [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm). Example: `{"bar-key": "value"}`
        """
        return pulumi.get(self, "freeform_tags")

    @property
    @pulumi.getter(name="fullRefName")
    def full_ref_name(self) -> str:
        """
        Unique full reference name inside a repository.
        """
        return pulumi.get(self, "full_ref_name")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="objectId")
    def object_id(self) -> str:
        """
        SHA-1 hash value of the object pointed to by the tag.
        """
        return pulumi.get(self, "object_id")

    @property
    @pulumi.getter(name="refName")
    def ref_name(self) -> str:
        """
        Unique reference name inside a repository.
        """
        return pulumi.get(self, "ref_name")

    @property
    @pulumi.getter(name="refType")
    def ref_type(self) -> str:
        """
        The type of reference (BRANCH or TAG).
        """
        return pulumi.get(self, "ref_type")

    @property
    @pulumi.getter(name="repositoryId")
    def repository_id(self) -> str:
        """
        The OCID of the repository containing the reference.
        """
        return pulumi.get(self, "repository_id")


class AwaitableGetRepositoryRefResult(GetRepositoryRefResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetRepositoryRefResult(
            commit_id=self.commit_id,
            defined_tags=self.defined_tags,
            freeform_tags=self.freeform_tags,
            full_ref_name=self.full_ref_name,
            id=self.id,
            object_id=self.object_id,
            ref_name=self.ref_name,
            ref_type=self.ref_type,
            repository_id=self.repository_id)


def get_repository_ref(ref_name: Optional[str] = None,
                       repository_id: Optional[str] = None,
                       opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetRepositoryRefResult:
    """
    This data source provides details about a specific Repository Ref resource in Oracle Cloud Infrastructure Devops service.

    Retrieves a repository's reference by its name with preference for branches over tags if the name is ambiguous. This can be disambiguated by using full names like "heads/<name>" or "tags/<name>".

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_repository_ref = oci.DevOps.get_repository_ref(ref_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        repository_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str ref_name: A filter to return only resources that match the given reference name.
    :param str repository_id: Unique repository identifier.
    """
    __args__ = dict()
    __args__['refName'] = ref_name
    __args__['repositoryId'] = repository_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('oci:DevOps/getRepositoryRef:getRepositoryRef', __args__, opts=opts, typ=GetRepositoryRefResult).value

    return AwaitableGetRepositoryRefResult(
        commit_id=__ret__.commit_id,
        defined_tags=__ret__.defined_tags,
        freeform_tags=__ret__.freeform_tags,
        full_ref_name=__ret__.full_ref_name,
        id=__ret__.id,
        object_id=__ret__.object_id,
        ref_name=__ret__.ref_name,
        ref_type=__ret__.ref_type,
        repository_id=__ret__.repository_id)


@_utilities.lift_output_func(get_repository_ref)
def get_repository_ref_output(ref_name: Optional[pulumi.Input[str]] = None,
                              repository_id: Optional[pulumi.Input[str]] = None,
                              opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetRepositoryRefResult]:
    """
    This data source provides details about a specific Repository Ref resource in Oracle Cloud Infrastructure Devops service.

    Retrieves a repository's reference by its name with preference for branches over tags if the name is ambiguous. This can be disambiguated by using full names like "heads/<name>" or "tags/<name>".

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_repository_ref = oci.DevOps.get_repository_ref(ref_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        repository_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str ref_name: A filter to return only resources that match the given reference name.
    :param str repository_id: Unique repository identifier.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

from .. import _utilities
import typing
# Export this package's modules as members:
from .get_protected_database import *
from .get_protected_database_fetch_configuration import *
from .get_protected_databases import *
from .get_protection_policies import *
from .get_protection_policy import *
from .get_recovery_service_subnet import *
from .get_recovery_service_subnets import *
from .protected_database import *
from .protection_policy import *
from .recovery_service_subnet import *
from ._inputs import *
from . import outputs
